from pathlib import Path

DOCS = "https://PKGNAME.readthedocs.io/en/latest/"
GIT_URL = "https://github.com/SouthGreenPlatform/PKGNAME"

SINGULARITY_URL_FILES = [('oras://registry.forge.ird.fr/diade/culebront_pipeline/apptainer/apptainer.culebront_tools.sif:0.0.1',
                          f'INSTALL_PATH/containers/apptainer.culebront_tools.sif')
                         ]

DATATEST_URL_FILES = ("https://itrop.ird.fr/culebront_utilities/Data-Xoo-sub.zip", "Data-Xoo-sub.zip")



